
function gadget:GetInfo()
	return {
		name      = 'Ore Economy',
		desc      = 'Continuously spawns ore on mex spots, changes mexes into ore extractors and makers into ore purifiers',
		author    = 'zerver',
		version   = 'v1.1',
		date      = 'Jan 2012',
		license   = 'GNU GPL, v2 or later',
		layer     = 0,
		enabled   = true
	}
end

----------------------------------------------------------------
-- Synced only
----------------------------------------------------------------
if not gadgetHandler:IsSyncedCode() then

local updatedMexes = {}
local allMexes = {}
local updatedMakers = {}
local allMakers = {}
local updatedOre = {}
local allOre = {}
local spAreTeamsAllied = Spring.AreTeamsAllied
local spGetMyTeamID = Spring.GetMyTeamID
local spGetSpectatingState = Spring.GetSpectatingState
local specFullView = false
local myTeam = -1

local function MexUpdate(unitID, unitTeam, eff, cap, ore)
	if specFullView or spAreTeamsAllied(myTeam, unitTeam) then
		updatedMexes[unitID] = {eff, cap, ore}
	end
end

local function MakerUpdate(unitID, unitTeam, eff, cap)
	if specFullView or spAreTeamsAllied(myTeam, unitTeam) then
		updatedMakers[unitID] = {eff, cap}
	end
end

local function OreUpdate(unitID, unitTeam, ore)
	if specFullView or spAreTeamsAllied(myTeam, unitTeam) then
		updatedOre[unitID] = ore
	end
end

local function UpdateMex(_, unitID, unitTeam, eff, cap, ore)
	if eff > 0 then
		allMexes[unitID] = {unitTeam, eff, cap, ore}
	else
		allMexes[unitID] = nil
	end
	MexUpdate(unitID, unitTeam, eff, cap, ore)
end

local function UpdateMaker(_, unitID, unitTeam, eff, cap)
	if eff > 0 then
		allMakers[unitID] = {unitTeam, eff, cap}
	else
		allMakers[unitID] = nil
	end
	MakerUpdate(unitID, unitTeam, eff, cap)
end

local function UpdateOre(_, unitID, unitTeam, ore)
	if ore > 0 then
		allOre[unitID] = {unitTeam, ore}
	else
		allOre[unitID] = nil
	end
	OreUpdate(unitID, unitTeam, ore)
end

local function UpdateTeamStatus()
	local _, spFullView, _ = spGetSpectatingState()
	local mTeam = spGetMyTeamID()
	if specFullView ~= spFullView or myTeam ~= mTeam then
		specFullView = spFullView
		myTeam = mTeam
		
		updatedMexes = {}
		for unitID, uData in pairs(allMexes) do
			MexUpdate(unitID, uData[1], uData[2], uData[3], uData[4])
		end
		Script.LuaUI.OEFullMexUpdate(updatedMexes)
		updatedMexes = {}

		updatedMakers = {}
		for unitID, uData in pairs(allMakers) do
			MakerUpdate(unitID, uData[1], uData[2], uData[3])
		end
		Script.LuaUI.OEFullMakerUpdate(updatedMakers)
		updatedMakers = {}
		
		updatedOre = {}
		for unitID, uData in pairs(allOre) do
			OreUpdate(unitID, uData[1], uData[2])
		end
		Script.LuaUI.OEFullOreUpdate(updatedOre)
		updatedOre = {}
	end
end

local function Update(_, n)
	if next(updatedMexes) ~= nil then
		Script.LuaUI.OEMexUpdate(updatedMexes)
		updatedMexes = {}
	end
	if next(updatedMakers) ~= nil then
		Script.LuaUI.OEMakerUpdate(updatedMakers)
		updatedMakers = {}
	end
	if next(updatedOre) ~= nil then
		Script.LuaUI.OEOreUpdate(updatedOre)
		updatedOre = {}
	end
end

local function UpdateGUI(_, force)
	if force == "1" then
		specFullView = nil
	end
	UpdateTeamStatus()
end

function gadget:Initialize()
	_, specFullView, _ = spGetSpectatingState()
	myTeam = spGetMyTeamID()
	gadgetHandler:AddSyncAction("oe_mex", UpdateMex)
	gadgetHandler:AddSyncAction("oe_mkr", UpdateMaker)
	gadgetHandler:AddSyncAction("oe_ore", UpdateOre)
	gadgetHandler:AddSyncAction("oe_upd", Update)
	gadgetHandler:AddChatAction("oreecongui", UpdateGUI, "requests data to be broadcasted to the ore econ gui")
end

function gadget:Shutdown()
	gadgetHandler:RemoveChatAction("oreecongui")
	gadgetHandler:RemoveSyncAction("oe_upd")
	gadgetHandler:RemoveSyncAction("oe_ore")
	gadgetHandler:RemoveSyncAction("oe_mkr")
	gadgetHandler:RemoveSyncAction("oe_mex")
end

else ----------------------------------------------------------------

local gaiaTeamID = Spring.GetGaiaTeamID()
local metalCoords = {}
local metalFactor = 0
local mexUpdateTeams = {}
local mexExp = {}
local lastMexSpeed = {}
local orePerUnit = {}
local oreDefName = "ore"
local addedUnits = {}
local mmecon = true
local updore = 0
local econLevelRegex = '^' .. string.char(137) .. '(%d+)$'
local teamEconLevels = {}
local lastTeamEnergyInfo = {}
local lastSwitch = {}
local oreDefID = FeatureDefNames[oreDefName].id
local oreSearchRad = Game.extractorRadius
local msx, msz = Spring.GetMetalMapSize()
local spAddUnitResource = Spring.AddUnitResource
local spUseUnitResource = Spring.UseUnitResource
local spCallCOBScript = Spring.CallCOBScript
local spSetFeatureReclaim = Spring.SetFeatureReclaim
local spDestroyFeature = Spring.DestroyFeature
local spCreateFeature = Spring.CreateFeature
local spGetFeatureResources = Spring.GetFeatureResources
local spGetUnitPosition = Spring.GetUnitPosition
local spGetTeamResources = Spring.GetTeamResources
local spGetFeatureDefID = Spring.GetFeatureDefID
local spGetFeaturesInCylinder = Spring.GetFeaturesInCylinder
local spGetFeaturesInRectangle = Spring.GetFeaturesInRectangle
local spGetFeaturePosition = Spring.GetFeaturePosition
local spGetGroundHeight = Spring.GetGroundHeight
local spSetFeatureDirection = Spring.SetFeatureDirection
local spGetUnitDefID = Spring.GetUnitDefID
local spGetUnitMetalExtraction = Spring.GetUnitMetalExtraction
local spGetUnitHealth = Spring.GetUnitHealth
local spGetMetalAmount = Spring.GetMetalAmount
local spGetUnitTeam = Spring.GetUnitTeam
local spGetGameFrame = Spring.GetGameFrame
local spGetUnitsInCylinder = Spring.GetUnitsInCylinder
local spGetUnitIsActive = Spring.GetUnitIsActive
local spGetUnitIsStunned = Spring.GetUnitIsStunned
local spGetUnitExperience = Spring.GetUnitExperience
local spGetCommandQueue = Spring.GetCommandQueue
local spGetUnitStates = Spring.GetUnitStates
local spGiveOrderToUnit = Spring.GiveOrderToUnit
local spSetUnitWeaponState = Spring.SetUnitWeaponState
local spGetUnitIsActive = Spring.GetUnitIsActive
local spGiveOrderToUnitMap = Spring.GiveOrderToUnitMap
local spSetTeamRulesParam = Spring.SetTeamRulesParam
local m_sqrt = math.sqrt
local m_min = math.min
local m_max = math.max
local m_pow = math.pow
local m_random = math.random
local m_pi = math.pi
local m_sin = math.sin
local m_cos = math.cos
local m_floor = math.floor
local m_ceil = math.ceil
local m_abs = math.abs

local makers = {
        [UnitDefNames.armmakr.id]  = { dist = 48, eff = 0.5, cap = 0.25},
        [UnitDefNames.cormakr.id]  = { dist = 54, eff = 0.5, cap = 0.25}, -- core T1 MM is bigger than arm
        [UnitDefNames.armfmkr.id]  = { dist = 48, eff = 0.5, cap = 0.25},
        [UnitDefNames.corfmkr.id]  = { dist = 48, eff = 0.5, cap = 0.25},
        [UnitDefNames.armmmkr.id]  = { dist = 72, eff = 1.5, cap = 0.25},
        [UnitDefNames.cormmkr.id]  = { dist = 72, eff = 1.5, cap = 0.25},
        [UnitDefNames.armuwmmm.id] = { dist = 72, eff = 1.5, cap = 0.25},
        [UnitDefNames.coruwmmm.id] = { dist = 72, eff = 1.5, cap = 0.25},
}
local makersPerTeam = {}

local mexes = {
        [UnitDefNames.armmex.id]  = { exp = false, nrg = 60, eff = 1, scr = "fixGo"},
        [UnitDefNames.cormex.id]  = { exp = false, nrg = 60, eff = 1, scr = "fixGo"},
        [UnitDefNames.armuwmex.id]  = { exp = false, nrg = 55, eff = 1, scr = "fixGo"},
        [UnitDefNames.coruwmex.id]  = { exp = false, nrg = 55, eff = 1, scr = "fixGo"},
        [UnitDefNames.armdev.id]  = { exp = true, nrg = 60, eff = 1, scr = "fixGo"},
        [UnitDefNames.corexp.id]  = { exp = true, nrg = 60, eff = 1, scr = "fixGo"},
        [UnitDefNames.armmoho.id]  = { exp = false, nrg = 50, eff = UnitDefNames.armmex.extractsMetal / UnitDefNames.armmoho.extractsMetal, scr = "fixGo"},
        [UnitDefNames.cormoho.id]  = { exp = false, nrg = 50, eff = UnitDefNames.cormex.extractsMetal / UnitDefNames.cormoho.extractsMetal, scr = "fixedGo"},
        [UnitDefNames.armuwmme.id]  = { exp = false, nrg = 50, eff = UnitDefNames.armuwmex.extractsMetal / UnitDefNames.armuwmme.extractsMetal, scr = "fixGo"},
        [UnitDefNames.coruwmme.id]  = { exp = false, nrg = 50, eff = UnitDefNames.coruwmex.extractsMetal / UnitDefNames.coruwmme.extractsMetal, scr = "fixGo"},
        [UnitDefNames.cormexp.id]  = { exp = true, nrg = 40, eff = UnitDefNames.cormex.extractsMetal / UnitDefNames.cormexp.extractsMetal, scr = "fixedGo"},
        [UnitDefNames.armmdev.id]  = { exp = true, nrg = 40, eff = UnitDefNames.armmex.extractsMetal / UnitDefNames.armmdev.extractsMetal, scr = "Go"},
}
local mexesPerTeam = {}


local function AddOre(x,y,z,ma)
	x = x * 16 + 8
	z = z * 16 + 8

	local feat = spGetFeaturesInRectangle(x, z, x, z)
  
	for f = 1, #feat do
		local ft = feat[f]
		if spGetFeatureDefID(ft) == oreDefID then
			local _,_,_,_,r = spGetFeatureResources(ft)
			spSetFeatureReclaim(ft, r + ma)
			return true
		end
	end
  
	local ft = spCreateFeature(oreDefID, x, y, z, 0, gaiaTeamID)
    
	if ft ~= nil and ft > 0 then
		spSetFeatureReclaim(ft, ma)
		local rd = m_random(360) * m_pi / 180
		spSetFeatureDirection(ft,m_sin(rd),0,m_cos(rd))
		return true
	end

	return false
end


function MultiOre(x, y, z, ma)
	local nMetal = m_max(1, m_min(10, m_ceil(ma / 100)))
	AddOre(m_max(0, m_min(m_floor(x / 16), msx - 1)), y,
		m_max(0, m_min(m_floor(z / 16), msz - 1)), ma / nMetal)
	if nMetal >= 2 then
		for i = 2, nMetal do
			AddOre(m_max(0, m_min(m_floor((x - 16 + m_random(32)) / 16), msx - 1)), y,
			m_max(0, m_min(m_floor((z - 16 + m_random(32)) / 16), msz - 1)), ma / nMetal)
		end
	end
end

function OreChanged(unitID, unitTeam, ore)
	SendToUnsynced("oe_ore", unitID, unitTeam, ore)
end

function gadget:Initialize()
	GG.orePerUnit = orePerUnit
	GG.addOre = AddOre
	GG.multiOre = MultiOre
	GG.oreChanged = OreChanged
	
	for x = 0, msx do
		for z = 0, msz do
			local ma = spGetMetalAmount(x, z)
			if ma > 0 then
				local mData = { x, z, ma, 0 }
				metalCoords[#metalCoords + 1] = mData
			end
		end
	end
	
	metalFactor = #metalCoords * 1000 * UnitDefNames.armmex.extractsMetal / 32
	
	local au = Spring.GetAllUnits()
	for _,unitID in ipairs(au) do
		local _, _, _, _, buildProg = spGetUnitHealth(unitID)
		if buildProg == 1 then
			self:UnitFinished(unitID, spGetUnitDefID(unitID), spGetUnitTeam(unitID))
		end
	end
end


function gadget:GameFrame(n)
	if #mexUpdateTeams > 0 then
    		local mutNew = {}
    		for t = 1, #mexUpdateTeams do
    			local uData = mexUpdateTeams[t]
    			if uData[3] < n then
				local units = spGetUnitsInCylinder(uData[1], uData[2], 2 * oreSearchRad + 64)
				for _, uID in ipairs(units) do
					local uTeam = spGetUnitTeam(uID)
					mpt = mexesPerTeam[uTeam]
					if mpt ~= nil then
						local mexData = mpt[uID]
						if mexData ~= nil and mexData.active then
							local newExt = spGetUnitMetalExtraction(uID)
							if mexData.mex ~= newExt then
								mexData.mex = newExt
								local mexTypeData = mexes[mexData.def]
								local mexEff = mexTypeData.eff
								local exp = mexTypeData.exp and mexExp[uID][2] or 1
								SendToUnsynced("oe_mex", uID, uTeam, exp * mexData.eff / mexEff, mexEff * mexData.mex * 1000 * mexData.cap, mexData.ore)
							end
						end
					end
				end
			else
				mutNew[#mutNew + 1] = uData
			end
    		end
	    	mexUpdateTeams = mutNew
    	end
    
    	if (n % 32) == 0 then
		mmecon = not(mmecon)
		updore = (updore + 1) % 4
	end
    	local curTeamMod = (n % 32)
    	for curTeam, mpt in pairs(mexesPerTeam) do
		if curTeamMod == (curTeam % 32) then
			local eLeft, eStor, ePull, eInc, eExp, eShare, eSent, eRecv, eExc = spGetTeamResources(curTeam, "energy")
			local energyLeft = eLeft
			local ooe = false

			for uID, mexData in pairs(mpt) do
				if mexData.active then
			  	  	local mexDefID = mexData.def
			  	  	local mexTypeData = mexes[mexDefID]
			  	  	local exp = mexTypeData.exp and mexExp[uID][2] or 1
				    	local purFactor = mexData.eff
					local effFactor = mexTypeData.eff
			  	  	local metalCap = effFactor * mexData.mex * 1000 * mexData.cap
			  	  	local eReqFactor, metalAdd, maxMetal = 0, 0, 0
			 	  	local eUpkeep = UnitDefs[mexDefID].energyUpkeep * 1000
			 	  	if purFactor > 1.01 then
					  	purFactor = purFactor * exp
			  	  		eReqFactor = mexTypeData.nrg * (purFactor - exp) / effFactor
				  	  	maxMetal = m_min(metalCap, (energyLeft - eUpkeep) / eReqFactor)
				  	else
				  		purFactor = exp
				  		maxMetal = metalCap
				  	end
				  	if eUpkeep ~= 0 and eUpkeep > energyLeft then
				  		maxMetal = 0
				  	end
				  	local _, stunned, _ = spGetUnitIsStunned(uID)
			 	  	if not(stunned) then
				  	  	local metalLeft = maxMetal
				  	  	local metalProd = metalCap 
				  	  	local feat = spGetFeaturesInCylinder(mexData.px, mexData.pz, oreSearchRad)
				  	  	if updore == 0 then
				  	  		local totore = 0
							for f = 1, #feat do
								local ft = feat[f]
								if spGetFeatureDefID(ft) == oreDefID then
									local r,_,_,_,_ = spGetFeatureResources(ft)
									totore = totore + r
								end
							end
							mexData.ore = totore
							SendToUnsynced("oe_mex", uID, curTeam, exp * mexData.eff / effFactor, metalCap, totore)							
				  	  	end
						for f = 1, #feat do
							if metalProd <= 0 then
								break
							end

							local ft = feat[f]
							if spGetFeatureDefID(ft) == oreDefID then
								local r,_,_,_,rc = spGetFeatureResources(ft)
								local mP
								if metalLeft > 0 then
									if r > metalLeft then
										spSetFeatureReclaim(ft, rc * (1 - metalLeft / r))
										metalLeft = 0
									else
										spDestroyFeature(ft)
										metalLeft = metalLeft - r
									end
								end
								if metalProd > 0 then
									if r > metalProd then
										metalProd = 0
									else
										metalProd = metalProd - r
									end
								end
							end
						end
						local mP = maxMetal - metalLeft
						metalAdd = mP * purFactor / effFactor
						local energyUse = mP * eReqFactor
						energyLeft = energyLeft - energyUse
						if metalAdd > 0 then
							energyUse = energyUse + eUpkeep
							spAddUnitResource(uID, "m",  metalAdd)
						end
						if energyUse > 0 then
							if not(spUseUnitResource(uID, "e", energyUse)) then -- safeguard against roundoff errors
								local el, _ = spGetTeamResources(curTeam, "energy")
								spUseUnitResource(uID, "e", el)
								energyLeft = 0						
							end
						end
						if maxMetal < metalCap then
							local energyWanted = ((metalCap - metalProd) - mP) * eReqFactor
							if energyWanted > 0 then
								spUseUnitResource(uID, "e", energyWanted)
								ooe = true
							end
						end

					end
					if m_abs(lastMexSpeed[uID] - metalAdd) > 0.01 then
  						spCallCOBScript(uID, "SetSpeed", 0, metalAdd * 100)
  						spCallCOBScript(uID, mexTypeData.scr, 0)
  						lastMexSpeed[uID] = metalAdd
  					end
				end
			end
			if ePull > eExp then
				eLeft = eLeft - (ePull - eExp)
			end
			if eExc > 0 then
				eLeft = eLeft + eExc
			end
			local energyDiff = (eInc + eRecv) - (eExp + eSent)
		--	local ltei = (lastTeamEnergyInfo[curTeam] ~= nil) and lastTeamEnergyInfo[curTeam] or { eLeft, eLeft, energyDiff }
			local ltei = (lastTeamEnergyInfo[curTeam] ~= nil) and lastTeamEnergyInfo[curTeam] or energyDiff
		--	local energyDiff = (eLeft - ltei[1])
			if mmecon then
				if lastSwitch[curTeam] == nil then
		--			energyDiff = energyDiff / 2.0 + (ltei[1] - ltei[2]) / 2.0
					energyDiff = energyDiff / 2.0 + ltei / 2.0
				end
				lastSwitch[curTeam] = nil
				local econLevel = (teamEconLevels[curTeam] ~= nil) and teamEconLevels[curTeam] or 0.75
				local def = ooe or (eLeft / eStor < econLevel and (eLeft + 10 * energyDiff) / eStor < econLevel)
				local exc = not(def) and ((eLeft >= eStor) or (eLeft / eStor >= econLevel and (eLeft + 10 * energyDiff) / eStor >= econLevel))
				if econLevel > 0 and (def or exc) then
					local fac = def and 1 or -1
					local switchmk = {}
					local change = fac * ((econLevel * eStor - eLeft) / 10 - energyDiff)
					if change < 0.1 then
						change = 0.1
					end
					local mpt = makersPerTeam[curTeam]
					local mexaff = {}
					if mpt ~= nil then
						while change > 0 do
							local target = nil
							local targeteff, targetmet = 0, 0
							for uID, makerData in pairs(mpt) do
								if makerData.active == def and switchmk[uID] == nil and makerData.anrg > 0.001 and makerData.amet > 0.001 then
									local eff = def and (makerData.anrg / makerData.amet) or (makerData.amet / makerData.anrg)
									if eff > targeteff or (eff == targeteff and makerData.amet > targetmet) then
										target = uID
										targeteff = eff
										targetmet = makerData.amet
									end
								end							
							end
							if target == nil then
								break
							end
							local mx = mpt[target].mx
							if next(mx) ~= nil then
								local engdiff = 0
								for mID, mxData in pairs(mx) do
									local mexData = mexesPerTeam[curTeam][mID]
									local mxDataEff = ((mexaff[mID] ~= nil) and mexaff[mID].eff or 0)
									local mxDataCap = ((mexaff[mID] ~= nil) and mexaff[mID].cap or 0)
									local mexDataEff = mexData.eff - mxDataEff
									local mexDataCap = mexData.cap - mxDataCap
									local mexTypeData = mexes[mexData.def]
					 		 	  	local exp = mexTypeData.exp and mexExp[mID][2] or 1
									local energyDiff = mexData.mex * 1000 * mexTypeData.nrg * (mxData.mxeff * (mexDataCap - fac * mxData.mxcap) + mxData.mxcap * (mexDataEff - exp))
									engdiff = engdiff + energyDiff
									mexaff[mID] = { eff = mxDataEff + mxData.mxeff * fac, cap = mxDataCap + mxData.mxcap * fac }
								end
								change = change - engdiff
							end
							switchmk[target] = true
						end
					end
					if next(switchmk) ~= nil then
  						spGiveOrderToUnitMap(switchmk, CMD.ONOFF, { def and 0 or 1 }, {} )
						lastSwitch[curTeam] = true
  					end
  				end
			end
		--	lastTeamEnergyInfo[curTeam] = { eLeft, ltei[1] }
			lastTeamEnergyInfo[curTeam] = energyDiff
		end
	end
    
	if #metalCoords > 0 then
		local ra = m_random(#metalCoords)
		local mc = metalCoords[ra]
		local tot = mc[4] + mc[3] * metalFactor
		if tot >= 1 then
			AddOre(mc[1], spGetGroundHeight(mc[1] * 16 + 8, mc[2] * 16 + 8), mc[2], tot)
			metalCoords[ra][4] = 0
		else
			metalCoords[ra][4] = tot
		end
	end

	if (n % 30) == 0 then
		SendToUnsynced("oe_upd", n)
		for unitID, uData in pairs(mexExp) do
			local exp, _ = spGetUnitExperience(unitID)
			local mexData = mexesPerTeam[uData[3]][unitID]
			if mexData.active and uData[1] ~= exp  then
				uData[1] = exp
				exp = m_pow(1 + exp, 2) -- 1.0 experience quadruples the output
				uData[2] = exp
				local mexEff = mexes[mexData.def].eff
				SendToUnsynced("oe_mex", unitID, uData[3], exp * mexData.eff / mexEff, mexEff * mexData.mex * 1000 * mexData.cap, mexData.ore)
			end
		end
	end
end


function gadget:AllowFeatureBuildStep(builderID, builderTeam, featureID, featureDefID, step)
	if featureDefID == oreDefID and GG.OreMiner[builderID] then
		local _,_,_,_,r = spGetFeatureResources(featureID)
		orePerUnit[builderID] = orePerUnit[builderID] + r
		OreChanged(builderID, builderTeam, orePerUnit[builderID])
		spDestroyFeature(featureID)
		return false
	end
	return true
end


local function UpdateRange(unitID, unitDefID, eff)
	if unitDefID == UnitDefNames.armdev.id then
		spSetUnitWeaponState(unitID, 0, "range", 425 + 150 * (eff - 1.0))
	elseif unitDefID == UnitDefNames.corexp.id then
		spSetUnitWeaponState(unitID, 0, "range", 430 + 140 * (eff - 1.0))
	elseif unitDefID == UnitDefNames.armmdev.id then
		spSetUnitWeaponState(unitID, 0, "range", 1025 + 250 * (eff - 1.0))
	elseif unitDefID == UnitDefNames.cormexp.id then
		spSetUnitWeaponState(unitID, 0, "range", 650 + 125 * (eff - 1.0))
		spSetUnitWeaponState(unitID, 1, "range", 650 + 350 * (eff - 1.0))
	end
end


local function UpdateMaker(unitID, unitDefID, unitTeam, mdata, mkpt, fac)
	local nau = addedUnits[unitID] + fac
	if nau > 1 or nau < 0 then
		return
	end
	addedUnits[unitID] = nau

	local fx,_,fz = spGetUnitPosition(unitID)
	local mpt = mexesPerTeam[unitTeam]
	local teff, tcap, tnrg, tmet = 0, 0, 0, 0
	local amx = {}
	if mpt ~= nil then
		for uID, mexData in pairs(mpt) do
			local dist = m_sqrt(m_pow(mexData.px - fx,2) + m_pow(mexData.pz - fz,2))
			if dist <= 7 * mdata.dist then
				local eff = 1 / m_pow(m_max(mdata.dist, dist) / mdata.dist, 2)
				if fac ~= 0 then
					local eff2 = eff * fac
					local meff = eff2 * mdata.eff
					local mcap = eff2 * mdata.cap
					mexData.eff = mexData.eff + meff
					mexData.cap = mexData.cap + mcap
					local mkData = (fac > 0) and { mkeff = meff, mkcap = mcap } or nil
					mexData.mk[unitID] = mxData
				end
				if mexData.active then
					local mexTypeData = mexes[mexData.def]
					local mexEff = mexTypeData.eff
					local mexCap = mexEff * mexData.mex * 1000
					local neff = eff * mdata.eff
					local ncap = eff * mdata.cap
					amx[uID] = { mxeff = neff, mxcap = ncap }
					neff = neff / mexEff
					ncap = ncap * mexCap
					teff = teff + neff
					tcap = tcap + ncap
					local nmet = mexCap * neff
					tnrg = tnrg + nmet * mexTypeData.nrg
					tmet = tmet + nmet
					if fac ~= 0 then
--						Spring.Echo('EFF'..mexData.eff..' '..mexData.cap)
						local exp = mexTypeData.exp and mexExp[uID][2] or 1
						SendToUnsynced("oe_mex", uID, unitTeam, exp * mexData.eff / mexEff, mexCap * mexData.cap, mexData.ore)
						UpdateRange(uID, mexData.def, mexData.eff)
					end
				end
			end
		end
	end
	local makerData = { px = fx, pz = fz, def = unitDefID, aeff = teff, acap = tcap, anrg = tnrg, amet = tmet, mx = amx, active = (fac > 0) }
	mkpt[unitID] = makerData
	if fac > 0 then
		SendToUnsynced("oe_mkr", unitID, unitTeam, teff, tcap)
	elseif fac < 0 then
		SendToUnsynced("oe_mkr", unitID, unitTeam, -1, -1)
	end
end


local function UpdateMex(unitID, unitDefID, unitTeam, mxpt, fac)
	local nau = addedUnits[unitID] + fac
	if nau > 1 or nau < 0 then
		return
	end
	addedUnits[unitID] = nau
	
	local fx,_,fz = spGetUnitPosition(unitID)
	local teff, tcap = 1, 1
	local mext = spGetUnitMetalExtraction(unitID)
	local mexTypeData = mexes[unitDefID]
	local mexEff = mexTypeData.eff
	local mexCap = mexEff * mext * 1000
	local mpt = makersPerTeam[unitTeam]
	local amk = {}
	if mpt ~= nil then
		for uID, makerData in pairs(mpt) do
			local mdata = makers[makerData.def]
			local dist = m_sqrt(m_pow(makerData.px - fx,2) + m_pow(makerData.pz - fz,2))
			if dist <= 7 * mdata.dist then
				local eff = 1 / m_pow(m_max(mdata.dist, dist) / mdata.dist, 2)
				if fac ~= 0 then
					local eff2 = eff * fac
					local meff = eff2 * mdata.eff
					local mcap = eff2 * mdata.cap
					local mxData = (fac > 0) and { mxeff = meff, mxcap = mcap } or nil
					makerData.mx[unitID] = mxData
					meff = meff / mexEff
					mcap = mcap * mexCap
					makerData.aeff = makerData.aeff + meff
					makerData.acap = makerData.acap + mcap
					local mmet = mexCap * meff
					makerData.anrg = makerData.anrg + mmet * mexTypeData.nrg
					makerData.amet = makerData.amet + mmet
				end
				if makerData.active then
					local neff = eff * mdata.eff
					local ncap = eff * mdata.cap
					teff = teff + neff
					tcap = tcap + ncap				
					amk[uID] = { mkeff = neff, mkcap = ncap }
					if fac ~= 0 then
						SendToUnsynced("oe_mkr", uID, unitTeam, makerData.aeff, makerData.acap)
					end
				end
			end
		end
	end
	local mexData = { eff = teff, cap = tcap, def = unitDefID, mex = mext, ore = -1, px = fx, pz = fz, mk = amk, active = (fac > 0) }
	mxpt[unitID] = mexData
	local exp = mexTypeData.exp and mexExp[unitID][2] or 1
	if fac > 0 then
		SendToUnsynced("oe_mex", unitID, unitTeam, exp * teff / mexEff, mexCap * mexData.cap, mexData.ore)
	elseif fac < 0 then
		SendToUnsynced("oe_mex", unitID, unitTeam, -1, -1, -1)
	end
	UpdateRange(unitID, unitDefID, teff)
--	Spring.Echo('TEFF'..mxpt[unitID].eff..' '..mxpt[unitID].cap)		
end


local function AddUnit(unitID, unitDefID, unitTeam)
	if addedUnits[unitID] ~= nil then
		return
	end
	addedUnits[unitID] = 0

	local uact = spGetUnitIsActive(unitID)
	if mexes[unitDefID] ~= nil then
		local mpt = mexesPerTeam[unitTeam]
		if mpt == nil then
			mpt = {}
			mexesPerTeam[unitTeam] = mpt
		end
		if mexes[unitDefID].exp then
			local exp, _ = spGetUnitExperience(unitID)
			mexExp[unitID] = { exp, m_pow(1 + exp, 2), unitTeam }
		end
		UpdateMex(unitID, unitDefID, unitTeam, mpt, uact and 1 or 0)	
		lastMexSpeed[unitID] = -1	
	end
	local mdata = makers[unitDefID]
	if mdata ~= nil then
		local mpt = makersPerTeam[unitTeam]
		if mpt == nil then
			mpt = {}
			makersPerTeam[unitTeam] = mpt
		end
		UpdateMaker(unitID, unitDefID, unitTeam, mdata, mpt, uact and 1 or 0)
	end
end


local function RemoveUnit(unitID, unitDefID, unitTeam)
	if addedUnits[unitID] == nil then
		return
	end

	local mpt = mexesPerTeam[unitTeam]
	if mexes[unitDefID] ~= nil then
		local mexData = nil
		if mpt ~= nil then
			mexData = mpt[unitID]
			UpdateMex(unitID, unitDefID, unitTeam, mpt, 0 - addedUnits[unitID])	
			mpt[unitID] = nil
		end
		if mexData == nil then
			local fx,_,fz = spGetUnitPosition(unitID)
			mexData = { px = fx, pz = fz }
		end
		mexExp[unitID] = nil
		lastMexSpeed[unitID] = nil
		mexUpdateTeams[#mexUpdateTeams + 1] = { mexData.px, mexData.pz, spGetGameFrame() + 2 }
	end
	local mdata = makers[unitDefID]	
	mpt = makersPerTeam[unitTeam]
	if mdata ~= nil and mpt ~= nil then
		local udata = mpt[unitID]
		if udata ~= nil then
			UpdateMaker(unitID, unitDefID, unitTeam, mdata, udata, 0 - addedUnits[unitID])
			mpt[unitID] = nil
		end
	end

	addedUnits[unitID] = nil
end


function gadget:AllowCommand(unitID, unitDefID, teamID, cmdID, cmdParams, cmdOptions)
	if cmdID == CMD.ONOFF and #cmdParams >= 1 then
		local _, _, _, _, buildProg = spGetUnitHealth(unitID)
		if buildProg == 1 then
			if cmdParams[1] == 0 then
				if unitDefID == UnitDefNames.armdev.id or unitDefID == UnitDefNames.corexp.id or unitDefID == UnitDefNames.armmdev.id  or unitDefID == UnitDefNames.cormexp.id then
					local fs = spGetUnitStates(unitID)
					if fs.firestate ~= 0 or next(spGetCommandQueue(unitID)) ~= nil then
						return false
					end
  					spCallCOBScript(unitID, "SetSpeed", 0, 0)
  					spCallCOBScript(unitID, mexes[unitDefID].scr, 0)					
				end
				if addedUnits[unitID] == 1 then
					if mexes[unitDefID] ~= nil then
						UpdateMex(unitID, unitDefID, teamID, mexesPerTeam[teamID], -1)
					end
					if makers[unitDefID] ~= nil then
						UpdateMaker(unitID, unitDefID, teamID, makers[unitDefID], makersPerTeam[teamID], -1)
					end
				end
			elseif cmdParams[1] == 1 then
				if addedUnits[unitID] == 0 then
					if mexes[unitDefID] ~= nil then
						UpdateMex(unitID, unitDefID, teamID, mexesPerTeam[teamID], 1)
					end
					if makers[unitDefID] ~= nil then
						UpdateMaker(unitID, unitDefID, teamID, makers[unitDefID], makersPerTeam[teamID], 1)
					end
				end
			end
		end
	elseif cmdID == CMD.FIRE_STATE and #cmdParams >= 1 and (unitDefID == UnitDefNames.armdev.id or unitDefID == UnitDefNames.corexp.id or unitDefID == UnitDefNames.armmdev.id  or unitDefID == UnitDefNames.cormexp.id) then
		local _, _, _, _, buildProg = spGetUnitHealth(unitID)
		if buildProg == 1 then
			if cmdParams[1] ~= 0 and (not spGetUnitIsActive(unitID)) then
				spGiveOrderToUnit(unitID, CMD.ONOFF, {1}, {})
			end
		end
	end
	return true
end


function gadget:UnitCreated(unitID, unitDefID, unitTeam)
	orePerUnit[unitID] = 0
end


function gadget:UnitFinished(unitID, unitDefID, unitTeam)
	AddUnit(unitID, unitDefID, unitTeam)
end


function gadget:UnitGiven(unitID, unitDefID, newTeam, oldTeam)
	if addedUnits[unitID] ~= nil then
		RemoveUnit(unitID, unitDefID, oldTeam)
		AddUnit(unitID, unitDefID, newTeam)
	end
end

function gadget:UnitDestroyed(unitID, unitDefID, unitTeam)
	local ma = orePerUnit[unitID]
	if ma > 0 then
		local x,y,z = spGetUnitPosition(unitID)
		MultiOre(x, y, z, ma)
		orePerUnit[unitID] = 0
		OreChanged(unitID, unitTeam, 0)
	end
	
	RemoveUnit(unitID, unitDefID, unitTeam)
	orePerUnit[unitID] = nil
end

function gadget:RecvLuaMsg(msg, playerID)
	local newLevel = tonumber(msg:match(econLevelRegex))
	if newLevel and newLevel >= 0 and newLevel <= 100 then
		local _, _, playerIsSpec, playerTeam = Spring.GetPlayerInfo(playerID)
		if not playerIsSpec then
			newLevel = newLevel / 100
			spSetTeamRulesParam(playerTeam, 'opLevel', newLevel)
			teamEconLevels[playerTeam] = newLevel
			return true
		end
	end
end

end