CMD_ORE_MINER = 34711
CMD_DUMP_ORE = 34712

function gadget:GetInfo()
	return {
		name	= "UnitOreMiner",
		desc	= "Add a toggle for cons to be configured as and ore miner, and add a command for dumping the ore",
		author	= "zerver",
		date	= "Sep 2011",
		license	= "GPL",
		layer	= 20,
		enabled	= true
	}
end

--------------------------------------------------------------------------------
local oreMinerDefs = {}
--------------------------------------------------------------------------------

if (gadgetHandler:IsSyncedCode()) then

	local spGetCommandQueue = Spring.GetCommandQueue
	local spGiveOrderToUnit = Spring.GiveOrderToUnit
	local spFindUnitCmdDesc = Spring.FindUnitCmdDesc
	local spEditUnitCmdDesc = Spring.EditUnitCmdDesc
	local spGetUnitPosition = Spring.GetUnitPosition
	local spInsertUnitCmdDesc = Spring.InsertUnitCmdDesc
	local m_pow = math.pow

	local oreMinerUnits = {}

	local oreMinerActiveUnits = {}
	
	local inGiveCmd = false	

	local oreMinerCmdDesc = {
		id		= CMD_ORE_MINER,
		type	= CMDTYPE.ICON_MODE,
		name	= 'Ore Miner',
		cursor	= 'OreMiner',
		action	= 'oreminer',
		tooltip = 'Configure this unit to stockpile ore instead of purifying it into metal',
		params	= {'0', 'Ore Miner', 'Ore Miner' }
	}
	
	local dumpOreCmdDesc = {
		id		= CMD_DUMP_ORE,
		type	= CMDTYPE.ICON_MAP,
		name	= 'Dump Ore',
		cursor	= 'Unload units',
		action	= 'dumpore',
		tooltip = 'Dump all stockpiled ore, preferably in range of an extractor'
	}	

--------------------------------------------------------------------------------

	local function ValidateoreMinerDefs(mds)
		local newDefs = {}
		for udName,oreMinerData in pairs(mds) do
			local ud = UnitDefNames[udName]
			if (not ud) then
				Spring.Echo('Bad oreminer unit type: ' .. udName)
			else
				newDefs[ud.id] = ud
			end
		end
		return newDefs
	end

--------------------------------------------------------------------------------

	local function AddOreMinerCmdDesc(unitID, active)
		oreMinerCmdDesc.params[1] = (active and '1') or '0'
		local insertID = spFindUnitCmdDesc(unitID, CMD.RECLAIM) or 123456
		spInsertUnitCmdDesc(unitID, insertID + 1, oreMinerCmdDesc)
		spInsertUnitCmdDesc(unitID, insertID + 2, dumpOreCmdDesc)
	end

--------------------------------------------------------------------------------

	local function AddOreMinerUnit(unitID, oreMinerDef)

		local oreMinerData = {
			id = unitID,
			def = oreMinerDef,
			active	= false,
		}

		oreMinerUnits[unitID] = oreMinerData

		AddOreMinerCmdDesc(unitID, false)

		oreMinerActiveUnits[unitID] = nil
	end

--------------------------------------------------------------------------------

	function gadget:Initialize()

		-- Find all builders that can reclaim
		for id,_ in pairs(UnitDefs) do
			if UnitDefs[id].canReclaim and UnitDefs[id].canRestore then
				oreMinerDefs[UnitDefs[id].name] = {}
			end
		end

		if (not oreMinerDefs) then
			gadgetHandler:RemoveGadget()
			return
		end
		
		gadgetHandler:RegisterCMDID(CMD_ORE_MINER)
		gadgetHandler:RegisterCMDID(CMD_DUMP_ORE)
		
		Spring.SetCustomCommandDrawData(CMD_DUMP_ORE, "Unload units")

		oreMinerDefs = ValidateoreMinerDefs(oreMinerDefs)

		for _,unitID in ipairs(Spring.GetAllUnits()) do
			local unitDefID = Spring.GetUnitDefID(unitID)
			local oreMinerDef = oreMinerDefs[unitDefID]
			if (oreMinerDef) then
				AddOreMinerUnit(unitID, oreMinerDef)
			end
		end
		
		GG.OreMiner = oreMinerActiveUnits
	end

--------------------------------------------------------------------------------

	function gadget:Shutdown()
		for _,unitID in ipairs(Spring.GetAllUnits()) do
			local ud = UnitDefs[Spring.GetUnitDefID(unitID)]
			local cmdDescID = spFindUnitCmdDesc(unitID, CMD_ORE_MINER)
			if (cmdDescID) then
				Spring.RemoveUnitCmdDesc(unitID, cmdDescID)
			end
			local cmdDescID = spFindUnitCmdDesc(unitID, CMD_DUMP_ORE)
			if (cmdDescID) then
				Spring.RemoveUnitCmdDesc(unitID, cmdDescID)
			end
		end
	end

--------------------------------------------------------------------------------

	function gadget:UnitCreated(unitID, unitDefID, unitTeam, builderID)
		local oreMinerDef = oreMinerDefs[unitDefID]
		if oreMinerDef then
			AddOreMinerUnit(unitID, oreMinerDef)
		end
	end

--------------------------------------------------------------------------------

	function gadget:UnitDestroyed(unitID, unitDefID, unitTeam)
		oreMinerUnits[unitID] = nil
		oreMinerActiveUnits[unitID] = nil
	end

--------------------------------------------------------------------------------

	function OreMinerCommand(unitID, cmdParams)
		if (type(cmdParams[1]) ~= 'number') then
			return false
		end
		local oreMinerData = oreMinerUnits[unitID]
		if (not oreMinerData) then
			return false
		end

		local state = (cmdParams[1] == 1)
		if (state) then
			oreMinerActiveUnits[unitID] = true
		else
			oreMinerActiveUnits[unitID] = nil
		end
		oreMinerData.active = state

		local cmdDescID = spFindUnitCmdDesc(unitID, CMD_ORE_MINER)
		if (cmdDescID) then
			oreMinerCmdDesc.params[1] = (state and '1') or '0'
			spEditUnitCmdDesc(unitID, cmdDescID, { params = oreMinerCmdDesc.params})
		end
	end

--------------------------------------------------------------------------------

	function DumpOreCommand(unitID, unitTeam, cmdParams, cmdOptions, allowCmd)
		if inGiveCmd == false and #cmdParams >= 3 then
			local cx, cy, cz = cmdParams[1], cmdParams[2], cmdParams[3]
			
			if #cmdParams > 3 then
				
				if allowCmd == false then
					local cmds = spGetCommandQueue(unitID)
					local cmd = cmds[2]
					inGiveCmd = true
					if cmd ~= nil and cmd.id == CMD.RESTORE and cmd.params[1] == cx and cmd.params[2] == cy and cmd.params[3] == cz and cmd.params[4] == 0 then
						if GG.orePerUnit[unitID] <= 0 then
							spGiveOrderToUnit(unitID, CMD.REMOVE, {cmd.tag}, {})
						end
					elseif GG.orePerUnit[unitID] > 0 then
						spGiveOrderToUnit(unitID, CMD.INSERT, {cmds[1].tag, CMD.RESTORE, CMD.OPT_SHIFT, cx, cy, cz, 0}, {"right", "shift"})
					end
					inGiveCmd = false
    					return true	
				end
				return false
				
			else
				
				if allowCmd == true then
					local opt = {}
    					if (cmdOptions.shift) then table.insert(opt,"shift") end
					inGiveCmd = true	
					spGiveOrderToUnit(unitID, CMD_DUMP_ORE, {cx, cy, cz, 0}, opt)
					spGiveOrderToUnit(unitID, CMD.INSERT, {-1, CMD_DUMP_ORE, CMD.OPT_SHIFT, cx, cy, cz}, {"alt", "shift"})
					inGiveCmd = false
				end
				
				if GG.orePerUnit[unitID] <= 0 then
					return true
				end
				
				local x, _, z = spGetUnitPosition(unitID)
				local distsq = m_pow(cx - x, 2) + m_pow(cz - z, 2)
				local bd = oreMinerUnits[unitID].def.buildDistance
				if distsq <= bd * bd and allowCmd == false then
					GG.multiOre(cx, cy, cz, GG.orePerUnit[unitID])
					GG.orePerUnit[unitID] = 0
					GG.oreChanged(unitID, unitTeam, 0)
					return true
				end
				return false
				
			end
		end
		return false
	end
	
--------------------------------------------------------------------------------

	function gadget:AllowCommand(unitID, unitDefID, teamID, cmdID, cmdParams, cmdOptions)
		if (cmdID == CMD_ORE_MINER) then
			OreMinerCommand(unitID, cmdParams)
			return false
		elseif (cmdID == CMD_DUMP_ORE) then
			DumpOreCommand(unitID, teamID, cmdParams, cmdOptions, true)
			return (#cmdParams > 3)
		end
		return true
	end

--------------------------------------------------------------------------------

	function gadget:CommandFallback(unitID, unitDefID, teamID, cmdID, cmdParams, cmdOptions)
		if (cmdID == CMD_ORE_MINER) then
			return true, true
		elseif (cmdID == CMD_DUMP_ORE) then
			if DumpOreCommand(unitID, teamID, cmdParams, cmdOptions, false) then
				return true, true
			elseif inGiveCmd then
				return true, false
			end
		end
		return false
	end

--------------------------------------------------------------------------------

end
