
function widget:GetInfo()
  return {
    name      = "Ore Economy GUI",
    desc      = "Draws ore stockpile for miners and purification efficiency/capacity for extractors and purifiers",
    author    = "zerver",
    date      = "Jan 2012",
    license   = "GNU GPL, v2 or later",
    layer     = 10000,
    enabled   = true,
  }
end

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

local updatedMexes = {}
local updatedMakers = {}
local updatedOre = {}
local drawDistSq = 0
local spGetUnitPosition = Spring.GetUnitPosition
local spGetUnitHeight = Spring.GetUnitHeight
local spGetCameraPosition = Spring.GetCameraPosition
local spGetTimer = Spring.GetTimer
local spDiffTimers = Spring.DiffTimers
local glPushMatrix = gl.PushMatrix
local glPopMatrix = gl.PopMatrix
local glPushAttrib = gl.PushAttrib
local glPopAttrib = gl.PopAttrib
local glDepthTest = gl.DepthTest
local glText = gl.Text
local glTranslate = gl.Translate
local glBillboard = gl.Billboard
local glColor = gl.Color
local m_floor = math.floor
local s_format = string.format
local m_bits = math.bit_or(GL.CURRENT_BIT, GL.ENABLE_BIT)
local m_min = math.min
local m_max = math.max
local lastTime = spGetTimer()
local show = false
local firstUpdate = true

local function MexUpdate(upd)
	for unitID, uData in pairs(upd) do
		if uData[1] > 0 then
			updatedMexes[unitID] = {uData[1], uData[2], uData[3], spGetUnitHeight(unitID)}
		else
			updatedMexes[unitID] = nil
		end
	end
end

local function MakerUpdate(upd)
	for unitID, uData in pairs(upd) do
		if uData[1] > 0 then
			updatedMakers[unitID] = {uData[1], uData[2], spGetUnitHeight(unitID)}
		else
			updatedMakers[unitID] = nil
		end
	end
end

local function OreUpdate(upd)
	for unitID, uData in pairs(upd) do
		if uData > 0 then
			updatedOre[unitID] = {uData, spGetUnitHeight(unitID)}
		else
			updatedOre[unitID] = nil
		end
	end
end

local function FullMexUpdate(upd)
	updatedMexes = {}
	for unitID, uData in pairs(upd) do
		updatedMexes[unitID] = {uData[1], uData[2], uData[3], spGetUnitHeight(unitID)}
	end
end

local function FullMakerUpdate(upd)
	updatedMakers = {}
	for unitID, uData in pairs(upd) do
		updatedMakers[unitID] = {uData[1], uData[2], spGetUnitHeight(unitID)}
	end
end

local function FullOreUpdate(upd)
	updatedOre = {}
	for unitID, uData in pairs(upd) do
		updatedOre[unitID] = {uData, spGetUnitHeight(unitID)}
	end
end

function widget:PlayerChanged(playedID)
	if Spring.GetGameFrame() > 0 then
		Spring.SendCommands({"luarules oreecongui 0"})
	end
end

function widget:TeamChanged(teamID)
	if Spring.GetGameFrame() > 0 then
		Spring.SendCommands({"luarules oreecongui 0"})
	end
end

function widget:Initialize()
	drawDistSq = Spring.GetConfigInt("UnitLodDist", 1000)
	drawDistSq = drawDistSq * drawDistSq
	widgetHandler:RegisterGlobal("OEMexUpdate", MexUpdate)
	widgetHandler:RegisterGlobal("OEFullMexUpdate", FullMexUpdate)
	widgetHandler:RegisterGlobal("OEMakerUpdate", MakerUpdate)
	widgetHandler:RegisterGlobal("OEFullMakerUpdate", FullMakerUpdate)
	widgetHandler:RegisterGlobal("OEOreUpdate", OreUpdate)
	widgetHandler:RegisterGlobal("OEFullOreUpdate", FullOreUpdate)
	widgetHandler:UpdateCallIn("PlayerChanged")
	widgetHandler:UpdateCallIn("TeamChanged")
end 

function widget:GameFrame(n)
	if firstUpdate and n > 0 then
		Spring.SendCommands({"luarules oreecongui 1"})
		firstUpdate = false
	end
end

function widget:DrawWorld()
	local cx, cy, cz = spGetCameraPosition()

	glPushAttrib(m_bits)
	glDepthTest(true)
	glPushMatrix()
	
	local curTime = spGetTimer()
	if spDiffTimers(curTime, lastTime) > 0.5 then
		lastTime = curTime
		show = not(show)
	end
	
	for unitID, uData in pairs(updatedMexes) do
		local px, py, pz = spGetUnitPosition(unitID)
		if pz ~= nil then
			local dx, dy, dz = ((px - cx) * (px - cx)), ((py - cy) * (py - cy)), ((pz - cz) * (pz - cz))

			if ((dx + dy + dz) < (drawDistSq * 500)) then
				glPushMatrix()
				glTranslate(px, py + uData[4] + 1.0, pz)
				glBillboard()
						
				glColor(1.0, 1.0, 1.0, 1.0)
				local cap = m_floor(uData[2] * 10 + 0.5) / 10
				glText(s_format("%g/s", cap), 0.0, 12.0, 12.0, "xco")
				local eff = m_floor(uData[1] * 10 + 0.5) / 10
				glText(s_format("x%g", eff), 0.0, 0.0, 12.0, "xco")
				local ore = m_floor(uData[3] + 0.5)
				if ore >= 0 then
--					cap = 30.0 * m_max(cap, 0.1)
--					local r = m_max(0.0, 1.0 - ore / cap)
--					local g = m_min(1.0, ore / cap) - m_min(1.0, m_max(0.0, 1.0 - (ore - cap) / cap))
--					local b = m_min(1.0, m_max(0.0, (ore - cap) / cap))
					if ore > 0 or show then
						glText((ore..''), 0.0, -12.0, 12.0, "xco")
					end
				end
				glPopMatrix()
			end
		end
	end

	for unitID, uData in pairs(updatedMakers) do
		local px, py, pz = spGetUnitPosition(unitID)
		if pz ~= nil then
			local dx, dy, dz = ((px - cx) * (px - cx)), ((py - cy) * (py - cy)), ((pz - cz) * (pz - cz))

			if ((dx + dy + dz) < (drawDistSq * 500)) then
				glPushMatrix()
				glTranslate(px, py + uData[3] + 1.0, pz)
				glBillboard()
						
				glColor(1.0, 1.0, 1.0, 1.0)
				local cap = m_floor(uData[2] * 10 + 0.5) / 10
				glText(s_format("(+%g/s)", cap), 0.0, 0.0, 6.0, "xco")
				local eff = m_floor(uData[1] * 10 + 0.5) / 10
				glText(s_format("(+x%g)", eff), 0.0, -6.0, 6.0, "xco")
				glPopMatrix()
			end
		end
	end
	
	for unitID, uData in pairs(updatedOre) do
		local px, py, pz = spGetUnitPosition(unitID)
		if pz ~= nil then
			local dx, dy, dz = ((px - cx) * (px - cx)), ((py - cy) * (py - cy)), ((pz - cz) * (pz - cz))

			if ((dx + dy + dz) < (drawDistSq * 500)) then
				glPushMatrix()
				glTranslate(px, py + uData[2] + 1.0, pz)
				glBillboard()
						
				glColor(1.0, 1.0, 1.0, 1.0)
				local ore = m_floor(uData[1] + 0.5)
				glText((ore..''), 0.0, -10.0, 10.0, "xco")
				glPopMatrix()
			end
		end
	end	

	glPopMatrix()
	glPopAttrib()
end
















